#include <Wire.h>
#include <LiquidCrystal_I2C.h>

LiquidCrystal_I2C lcd(0x27, 16, 2);

#define btn PD2
#define pump PD5
#define TRIG_PIN PD7
#define ECHO_PIN PD6

long ts = 0;

bool isButtonPressed = false;

void setup() {
  lcd.init();
  lcd.clear();         
  lcd.backlight();

  setup_interrupts();

  // Setarea pompei ca iesire
  DDRD |= (1 << pump);

  // Configurarea registrilor necesari pentru PWM
  // Setarea modului de funcționare PWM
  TCCR0A |= (1 << WGM01);
  // Setarea prescalerului pentru un factor de divizare de 64
  TCCR0B |= (1 << CS01) | (1 << CS00);
  // Setarea COM0B1 pentru a permite PWM non-inversat
  TCCR0A |= (1 << COM0B1);

  // Setarea TRIG_PIN ca output
  DDRD |= (1 << TRIG_PIN);
  // Setarea ECHO_PIN ca input
  DDRD &= ~(1 << ECHO_PIN);

}

void loop() {
  // Masurarea distantei folosind sensor ultrasonic 
  float distance = measureDistance();

 if (distance < 10 || isButtonPressed) {
    pump_is_on();
    lcd.setCursor(0, 0);
    lcd.print("Oski is drinking");
  } else {
    pump_is_off();
    lcd.setCursor(0, 0);
    lcd.print("                ");  // Clear the line
  }

  delay(1000);
}

ISR(PCINT2_vect) {
  char val = PIND & (1 << btn);
  if ((millis() - ts) >= 100) {
    ts = millis();
    if ((int) val == 0) {
      if (isButtonPressed) {
        // Închide pompa
        isButtonPressed = false;
        pump_is_off();
      } else {
        // Porneste pompa
        isButtonPressed = true;
        pump_is_on();
      }
    }
  }
}

// Porneste pompa
void pump_is_on() {
  OCR0B = 3 * 256 / 5;
}

// Închide pompa
void pump_is_off() {
  OCR0B = 0;
}

float measureDistance() {
  // Declanșarea senzorului ultrasonic
  PORTD &= ~(1 << TRIG_PIN);
  _delay_us(2);
  PORTD |= (1 << TRIG_PIN);
  _delay_us(10);
  PORTD &= ~(1 << TRIG_PIN);

  // Măsurarea duratei pulsului
  long duration = pulseIn(ECHO_PIN, HIGH);

  // Calcularea distanței în centimetri
  float distance = duration / 58.0;

  return distance;
}


// Afisarea mesajului pe LCD
void print_on_lcd () {
  lcd.setCursor(0,0);
  lcd.print("Oski is drinking");
}


void setup_interrupts() {
  cli();
 
  // Configureare pinului asociat butonului ca intrare
  DDRD &= ~(1 << btn);
  // Activare rezistența de pull-up pentru buton
  PORTD |= (1 << btn);
  // Activare întreruperi PCINT2 
  PCICR |= (1 << PCIE2); 
  // Activare întreruperi buton (PD2, PCINT18)
  PCMSK2 |= (1 << PCINT18);
  
  sei();
}